const db = require('../db/index');
const moment = require('moment');
const { Op } = require("sequelize");


const pay = async(customerId, date) => {
    try {
        console.log(customerId)
        let result = await db.customers.update({ 
            date
         },
         {
            where:{
                customerId
            }
         })
         console.log(result)
         return true;
    } catch (error) {
        console.log(error)
        return false;
    }
    
}

const updateCustomer = async(id, body) => {
    try {
        const date = moment(body.date, 'YYYY-MM-DD');
        await db.customers.update({
            nombres: body.nombres,
            apellidoPaterno: body.apellidoPaterno,
            apellidoMaterno: body.apellidoMaterno,
            telefono: body.telefono,
            correo: body.correo,
            correo: body.correo,
            evolutionId: parseInt(body.valkiriaID),
            userCategoryId: parseInt(body.category),
            date
        },
        {
            where:{
                customerId: id
            }
        })
        return true;
    } catch (error) {
        console.log(error)
        return false;
    }
}

const deleteCustoomer = async(id) => {
    try {
        await db.customers.destroy({
            where:{
                customerId: id
            }
        })
        return true;
    } catch (error) {
        return false;
    }
}

const createCustomer = async(body) => {
    try {
        console.log(body)
        const fechaFormateada = moment(body.date);
        console.log({
            nombres: body.nombres,
            apellidoPaterno: body.apellidoPaterno,
            apellidoMaterno: body.apellidoMaterno,
            telefono: body.telefono,
            correo: body.correo,
            evolutionId: parseInt(body.evolutionID),
            userCategoryId: parseInt(body.categoria),
            isActive: true,
            date: body.date
        })
        await db.customers.create({
            nombres: body.nombres,
            apellidoPaterno: body.apellidoPaterno,
            apellidoMaterno: body.apellidoMaterno,
            telefono: body.telefono,
            correo: body.correo,
            evolutionId: parseInt(body.evolutionID),
            userCategoryId: parseInt(body.categoria),
            isActive: true,
            date: body.date
        })
        return { result: true }
    } catch (error) {
        console.log(error)
        return { result: false }
    }
    
}

const getCustumers = async () => {
    let listToReturn = [];
    let customers = await db.customers.findAll(
        {
            include: [{
                model: db.userCategories,
                as: 'userCategory',
                required:false
            }]
        }
    );
    listToReturn = await proccessCustomers(customers);
    return listToReturn;
}


const getUsersByKeyword= async(keyWord) => {
    try {
        let listToReturn = [];
        let customers = await db.customers.findAll({
            include: [{
                model: db.userCategories,
                as: 'userCategory',
                required:false
            }],
            where:{
                [Op.or]: [
                    { nombres: { [Op.like]: `%${keyWord}%` } },
                    { apellidoPaterno: { [Op.like]: `%${keyWord}%` } },
                    { apellidoMaterno: { [Op.like]: `%${keyWord}%` } },  
                    { telefono: { [Op.like]: `%${keyWord}%` } },  
                    { correo: { [Op.like]: `%${keyWord}%` } },  
                    { evolutionId: { [Op.like]: `%${keyWord}%` } },            	
                ],
            }
        });
        console.log("customers",customers)
        listToReturn = await proccessCustomers(customers);
        return listToReturn;
    } catch (error) {
        console.log(error)
        return [];
    }
}


const getCustomerById = async(customerId) => {
    let listToReturn = [];
    let customers = await db.customers.findAll({
        include: [{
            model: db.userCategories,
            as: 'userCategory',
            required:false
        }],
        where:{
            customerId
        }
    });
    listToReturn = await proccessCustomers(customers);
    if(listToReturn && listToReturn.length > 0)
    {
        return listToReturn[0]
    }
    else
    {
        return {};
    }
}


const proccessCustomers = async (customers) => {
    let listToReturn = [];
    if(customers)
    {
        for(const customer of customers)
        {
            let obj = { 
                id: null,
                nombres: null, 
                apellidoPaterno: null,   
                apellidoMaterno: null,
                telefono: null,
                evolutionId: null,
                categoria:null,
                categoryId: null,
                fechaInicio: null,
                fechaTermino: null,
                status: null,
                correo:null,
                settingCinta: null,
                settingGrado: null,
                cinta: null,
                grado: null,
            }
            //console.log(customer.userCategory.cinta)
            obj.id = customer.customerId;
            obj.nombres = customer.nombres;
            obj.categoria = customer.userCategory.name;
            obj.categoryId = customer.userCategory.categoryId;
            obj.apellidoPaterno = customer.apellidoPaterno ?? null;
            obj.apellidoMaterno = customer.apellidoMaterno ?? null;
            obj.telefono = customer.telefono ?? null;
            obj.evolutionId = customer.evolutionId ?? null;
            obj.correo  = customer.correo ?? null;
            let fechaInicio = moment(customer.date)
            let fechaTermino = moment(fechaInicio).add(1, 'M');
            let now = moment();
            if(now.isSameOrBefore(fechaTermino, 'day'))
            {
                obj.status  = "corriente";
            }
            else
            {
                obj.status  = "vencido";
            }
            obj.fechaInicio = fechaInicio.locale('es').format('L');
            obj.fechaTermino = fechaTermino.locale('es').format('L');            
            obj.fechaInicioInptFormat = convertMomentDateToInputFormat(fechaInicio)
            obj.fechaTerminoInptFormat = convertMomentDateToInputFormat(fechaTermino)
            if(customer.userCategory)
            {
                obj.settingCinta = customer.userCategory.cinta;
                obj.settingGrado = customer.userCategory.grado;
            }
            listToReturn.push(obj)
        }
    }
    
    return listToReturn;
}

const convertMomentDateToInputFormat = (date) =>
{
    const year = date.format('YYYY');
    const month = date.format('MM');
    const day = date.format('DD');
    return `${year}-${month}-${day}`;
}



module.exports = {
    createCustomer,
    getCustumers,
    getUsersByKeyword,
    getCustomerById,
    pay,
    updateCustomer,
    deleteCustoomer
}